#!/usr/bin/env python
from fractions import Fraction
import random

from simfile import *

class NotesAppend(Notes):
    def append(self, start, notes):
        # If we're setting the region to itself, make a copy first, otherwise
        # it might end up infinitely looping
        if notes is self:
            notes = Notes(unicode(notes))
        for row in notes:
            self.notes.append((row[0] + start, row[1]))

class Streamer(object):
    
    l, d, u, r, L, R = xrange(6)
    
    def __init__(self):
        self.history = [(Streamer.l, Streamer.r)]
        self.direction = Streamer.L
        self.last_used_foot = Streamer.R
    
    def stream(self, count):
        notes = Notes('0000')
        i = 0
        
        while i < count:
            l, r = self.history[-1]
            choices = [Streamer.l, Streamer.d, Streamer.u, Streamer.r]
            row = ['0', '0', '0', '0']
            
            if self.last_used_foot == Streamer.L:
                choices.remove(Streamer.l)
                if i + 1 == count:
                    choices.remove(Streamer.d)
                    choices.remove(Streamer.u)
                elif self.direction == Streamer.L:
                    choices.remove(Streamer.d)
                else:
                    choices.remove(Streamer.u)
                try:
                    if self.history[-2][1] == self.history[-4][1]:
                        choices.remove(self.history[-2][1])
                except (IndexError, ValueError):
                    pass
                try:
                    r = random.choice(choices)
                except IndexError:
                    r = Streamer.r
                row[r] = '1'
            else:
                choices.remove(Streamer.r)
                if i + 1 == count:
                    choices.remove(Streamer.d)
                    choices.remove(Streamer.u)
                elif self.direction == Streamer.L:
                    choices.remove(Streamer.u)
                else:
                    choices.remove(Streamer.d)
                try:
                    if self.history[-2][0] == self.history[-4][0]:
                        choices.remove(self.history[-2][0])
                except (IndexError, ValueError):
                    pass
                try:
                    l = random.choice(choices)
                except IndexError:
                    l = Streamer.l
                row[l] = '1'
            
            self.history.append((l, r))
            self.last_used_foot = Streamer.L if self.last_used_foot == Streamer.R else Streamer.R
            notes.set_row(Fraction(i, 4), Notes(''.join(row)))
            i += 1
        
        self.direction = Streamer.R if self.direction == Streamer.L else Streamer.L
        self.history = self.history[-4:]
        return notes


def main():
    sm = Simfile('NFG Talents Mix 001.sm')
    novice = sm.get_chart(stepstype='dance-single', difficulty='Beginner')
    stream = NotesAppend('0000')
    last_beat = Fraction(32)
    sixteenth = Fraction(1, 4)
    streamer = Streamer()

    for beat, _ in novice.notes:
        print beat
        stream.append(last_beat, streamer.stream(4 * (beat - last_beat)))
        last_beat = beat

    sm.set_chart(stream, stepstype='dance-single', difficulty='Challenge')
    sm.save()


if __name__ == '__main__':
    main()